#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/timeb.h>
#include <unistd.h>

#include <asm/arch-sa1100/hijack.h>

#define FALSE (0)
#define TRUE  (1)
#define STOICH 14.7

#include "vfdlib.h"
#include "innovateSerial2.h"

#define SERIALDEVICE "/dev/ttyS1"

unsigned char screenbuf[EMPEG_SCREEN_BYTES] = {0,};
const char *hijack_menu[] = { "Wideband O2", NULL };
int displayfd = -1;
unsigned    buttons[9] = {8,
			IR_LEFT_BUTTON_PRESSED,
			IR_RIGHT_BUTTON_PRESSED,
			IR_TOP_BUTTON_PRESSED,
			IR_BOTTOM_BUTTON_PRESSED,
			IR_KNOB_PRESSED,
			IR_KNOB_RIGHT,
			IR_KNOB_LEFT
			};
float afrBuf[EMPEG_SCREEN_COLS] = {0,};
int afrBufPos=0;
struct timeb prevSampleTime, currentTime;
int sampleInterval;


hijack_geom_t geom = {8,24,32,100}, fullscreen = {0,EMPEG_SCREEN_ROWS-1,0,EMPEG_SCREEN_COLS-1};

int compareTime(struct timeb *curr, struct timeb *prev, int interval)
{struct timeb elapsed;
 
 long int tempInt;
 
 	elapsed.time=curr->time-prev->time;
 	if (curr->millitm >= prev->millitm)
 		elapsed.millitm=curr->millitm-prev->millitm;
 	else {
 		elapsed.time--;
 		elapsed.millitm=1000+elapsed.millitm-prev->millitm;
 	}
 	
 	tempInt=elapsed.time*1000+elapsed.millitm;

	if (tempInt>interval)
 		return TRUE;
	else
 		return FALSE;
}


void displayText(const char *text)
{
	if (ioctl(displayfd, EMPEG_HIJACK_DISPCLEAR) == -1) {
	  fprintf(stderr, "Unable to clear screen.\n");
	  exit(1);
	}
	if (ioctl(displayfd,EMPEG_HIJACK_DISPTEXT, text) == -1) {
		fprintf(stderr, "Unable to display text.\n");
		exit(1);
	}	
}

void drawAFR(float AFR, int x, int y)
{
	char tempString[5];
	int legendWidth;
	legendWidth=vfdlib_getTextWidth("Air/Fuel Ratio", 0);
	vfdlib_drawText(screenbuf, "Air/Fuel Ratio", x+2, y, 0, VFDSHADE_MEDIUM);
	vfdlib_drawOutlineRectangleClipped(screenbuf, x, y+vfdlib_getTextHeight(0)+2, x+legendWidth+4, y+vfdlib_getTextHeight(0)+vfdlib_getTextHeight(2)+2, VFDSHADE_MEDIUM);
	sprintf(tempString,"%.2f",AFR);
	vfdlib_drawText(screenbuf, tempString, x, y+vfdlib_getTextHeight(0)+4, 2, VFDSHADE_BRIGHT);
	
}

void drawLambda(float lambda, int x, int y)
{
	char tempString[5];
	int legendWidth;
	legendWidth=vfdlib_getTextWidth("Lambda", 0);
	vfdlib_drawText(screenbuf, "Lambda", x+3, y, 0, VFDSHADE_MEDIUM);
	vfdlib_drawOutlineRectangleClipped(screenbuf, x, y+vfdlib_getTextHeight(0)+2, x+legendWidth+6, y+vfdlib_getTextHeight(0)+vfdlib_getTextHeight(2)+2, VFDSHADE_MEDIUM);
	sprintf(tempString,"%.2f",lambda);
	vfdlib_drawText(screenbuf, tempString, x, y+vfdlib_getTextHeight(0)+4, 2, VFDSHADE_BRIGHT);
	
}

void drawAFRCompact(float afr, int x, int y, int font)
{char tempString[5];
	
	sprintf(tempString, "%.2f", afr);
	vfdlib_drawText(screenbuf, "AFR:", x, y, font, VFDSHADE_MEDIUM);
	vfdlib_drawText(screenbuf, tempString, x+vfdlib_getTextWidth("AFR: ", font), y, font, VFDSHADE_BRIGHT);
}

void drawLambdaCompact(float lambda, int x, int y, int font)
{char tempString[5];
	
	sprintf(tempString, "%.2f", lambda);
	vfdlib_drawText(screenbuf, "Lambda:", x, y, font, VFDSHADE_MEDIUM);
	vfdlib_drawText(screenbuf, tempString, x+vfdlib_getTextWidth("Lambda: ", font), y, font, VFDSHADE_BRIGHT);
}

int afrToY(int height, int center, int range, float afr)
{int yval;
	yval=height/2+(int)(((afr-center)/(float)range)*(float)height);
	return yval;
}

void drawGraph(int height, int center, int range)
{int pos, yval, xval=0;
 
	
	for (pos=afrBufPos; pos<EMPEG_SCREEN_COLS; pos++) {
		yval=((afrBuf[pos]-center)/range)*(float)height;
		vfdlib_drawPointClipped(screenbuf,xval,afrToY(height,center,range,afrBuf[pos]),VFDSHADE_BRIGHT);
		xval++;
	}
	for (pos=0; pos<afrBufPos; pos++) {
		yval=((afrBuf[pos]-center)/range)*(float)height;
		vfdlib_drawPointClipped(screenbuf,xval,afrToY(height,center,range,afrBuf[pos]),VFDSHADE_BRIGHT);
		xval++;
	}
	
	vfdlib_drawLineHorizClipped(screenbuf, afrToY(height,center,range,STOICH), 0,
		EMPEG_SCREEN_COLS, VFDSHADE_DIM);
}

void updateDisplay()
{
	if (ioctl(displayfd, EMPEG_HIJACK_DISPWRITE, screenbuf) == -1) {
    	fprintf(stderr, "Unable to update display.\n");
    	exit(1);
	}
}

int main(int argc, char **argv)
{
	int done = FALSE;
	int f=0;
	int viewMode=0;
	float afr=14.7;
	unsigned int buttonData;
	word wbData;
	
	if ((displayfd = open("/dev/display", O_RDWR)) == -1) {
		fprintf(stderr, "Unable to open display.\n");
		exit(1);
	}
	
	while (!done) {
		if (ioctl(displayfd, EMPEG_HIJACK_WAITMENU, hijack_menu) == -1) {
			fprintf(stderr, "Unable to bind into Hijack menu.\n");
			return 1;
		}
		
		sampleInterval=1500;
		
		prevSampleTime.time=0;
		prevSampleTime.millitm=0;
		
		vfdlib_registerFont("/empeg/lib/fonts/small.bf", 0);
		vfdlib_registerFont("/empeg/lib/fonts/medium.bf", 1);
		vfdlib_registerFont("/empeg/lib/fonts/large.bf", 2);
		
		if (ioctl(displayfd, EMPEG_HIJACK_BINDBUTTONS, buttons) == -1) {
			fprintf(stderr, "Unable to bind buttons.\n");
			return 1;
		}
		
		if (ioctl(displayfd,EMPEG_HIJACK_SETGEOM, &fullscreen) == -1) {
			fprintf(stderr, "Unable to set geometry\n");
			return 1;
		}
		
		//drawAFR(afr, 0, 5);
		
		if (serialInit(SERIALDEVICE) == -1) {
			fprintf(stderr, "Unable to open serial device\n");
			return 1;
		}
		
		
		done = FALSE;
		while (!done) {
			if (ioctl(displayfd, EMPEG_HIJACK_POLLBUTTONS, &buttonData) != -1) {
				switch (buttonData) {
					case IR_RIGHT_BUTTON_PRESSED:
						if (viewMode==0)
							viewMode=1;
						else
							viewMode=0;
						break;
					case IR_LEFT_BUTTON_PRESSED:
						if (f==1)
							f=0;
						else
							f=1;
						break;
					case IR_BOTTOM_BUTTON_PRESSED:
						break;
					case IR_KNOB_PRESSED:
						break;
					case IR_KNOB_RIGHT:
						if (sampleInterval < 3000)
							sampleInterval += 25;
						printf("%d\n", sampleInterval);
						break;
					case IR_KNOB_LEFT:
						if (sampleInterval > 25)
							sampleInterval -= 25;
						printf("%d\n", sampleInterval);
						break;
					case IR_TOP_BUTTON_PRESSED:
						done = TRUE;
						break;
				}
			}
			ftime(&currentTime);
			if (compareTime(&currentTime, &prevSampleTime, sampleInterval)) {
				prevSampleTime=currentTime;
				afrBuf[afrBufPos]=afr;
				afrBufPos++;
				if (afrBufPos>EMPEG_SCREEN_COLS)
					afrBufPos=0;
				
			}
			
			afr+=(((float)rand()/(float)RAND_MAX)/10-0.05);
			if (afr > 20) afr--;
			else if (afr < 8) afr++;
			
			vfdlib_fastclear(screenbuf);
			switch (viewMode) {
				case 0:
					drawAFRCompact(afr, 0, EMPEG_SCREEN_ROWS-vfdlib_getTextHeight(f), f);
					drawLambdaCompact(afr/14.7, EMPEG_SCREEN_COLS/2, EMPEG_SCREEN_ROWS-vfdlib_getTextHeight(f), f);
					drawGraph(EMPEG_SCREEN_ROWS-vfdlib_getTextHeight(f), 14.7, 14);
					break;
				case 1:
					drawAFR(afr, 0, 0);
					drawLambda(afr/14.7, EMPEG_SCREEN_COLS/2, 0);
					break;
			}
			updateDisplay();
		}
	}
	
	vfdlib_unregisterAllFonts();
	if (ioctl(displayfd,EMPEG_HIJACK_WAITMENU, NULL) == -1) {
		fprintf(stderr, "Unable to release menu system.\n");
		return 1;
	}
	return 0;
}

